@@ -18,6 +18,7 @@ import (
 	"github.com/mattn/go-isatty"
 
 	"github.com/sourcegraph/sourcegraph/lib/errors"
+	"github.com/sourcegraph/sourcegraph/lib/output"
 
 	batcheslib "github.com/sourcegraph/sourcegraph/lib/batches"
 	"github.com/sourcegraph/sourcegraph/lib/batches/template"
@@ -31,10 +32,14 @@ import (
 	"github.com/sourcegraph/src-cli/internal/batches/repozip"
 	"github.com/sourcegraph/src-cli/internal/batches/service"
 	"github.com/sourcegraph/src-cli/internal/batches/ui"
+	"github.com/sourcegraph/src-cli/internal/batches/watchdog"
 	"github.com/sourcegraph/src-cli/internal/batches/workspace"
 	"github.com/sourcegraph/src-cli/internal/cmderrors"
 )
 
+// We check for docker responsiveness every minute
+const dockerWatchDuration = 1 * time.Minute
+
 // batchExecutionFlags are common to batch changes that are executed both
 // locally and remotely.
 type batchExecutionFlags struct {
@@ -85,6 +90,7 @@ type batchExecuteFlags struct {
 	workspace     string
 	cleanArchives bool
 	skipErrors    bool
+	runAsRoot     bool
 
 	// EXPERIMENTAL
 	textOnly bool
@@ -137,7 +143,7 @@ func newBatchExecuteFlags(flagSet *flag.FlagSet, cacheDir, tempDir string) *batc
 
 	flagSet.BoolVar(
 		&caf.cleanArchives, "clean-archives", true,
-		"If true, deletes downloaded repository archives after executing batch spec steps.",
+		"If true, deletes downloaded repository archives after executing batch spec steps. Note that only the archives related to the actual repositories matched by the batch spec will be cleaned up, and clean up will not occur if src exits unexpectedly.",
 	)
 
 	flagSet.BoolVar(
@@ -152,6 +158,11 @@ func newBatchExecuteFlags(flagSet *flag.FlagSet, cacheDir, tempDir string) *batc
 
 	flagSet.BoolVar(verbose, "v", false, "print verbose output")
 
+	flagSet.BoolVar(
+		&caf.runAsRoot, "run-as-root", false,
+		"If true, forces all step containers to run as root.",
+	)
+
 	return caf
 }
 
@@ -177,37 +188,17 @@ func batchDefaultCacheDir() string {
 	if err != nil {
 		return ""
 	}
-
-	// Check if there's an old campaigns cache directory but not a new batch
-	// directory: if so, we should rename the old directory and carry on.
-	//
-	// TODO(campaigns-deprecation): we can remove this migration shim after June
-	// 2021.
-	old := path.Join(uc, "sourcegraph", "campaigns")
 	dir := path.Join(uc, "sourcegraph", "batch")
-	if _, err := os.Stat(dir); os.IsNotExist(err) {
-		if _, err := os.Stat(old); os.IsExist(err) {
-			// We'll just try to do this without checking for an error: if it
-			// fails, we'll carry on and let the normal cache directory handling
-			// logic take care of it.
-			os.Rename(old, dir)
-		}
-	}
 
 	return dir
 }
 
 // batchDefaultTempDirPrefix returns the prefix to be passed to ioutil.TempFile.
-// If one of the environment variables SRC_BATCH_TMP_DIR or
-// SRC_CAMPAIGNS_TMP_DIR is set, that is used as the prefix. Otherwise we use
-// "/tmp".
+// If the environment variable SRC_BATCH_TMP_DIR is set, that is used as the prefix.
+// Otherwise we use "/tmp".
 func batchDefaultTempDirPrefix() string {
-	// TODO(campaigns-deprecation): we can remove this migration shim in
-	// Sourcegraph 4.0.
-	for _, env := range []string{"SRC_BATCH_TMP_DIR", "SRC_CAMPAIGNS_TMP_DIR"} {
-		if p := os.Getenv(env); p != "" {
-			return p
-		}
+	if p := os.Getenv("SRC_BATCH_TMP_DIR"); p != "" {
+		return p
 	}
 
 	// On macOS, we use an explicit prefix for our temp directories, because
@@ -264,13 +255,34 @@ type executeBatchSpecOpts struct {
 	client api.Client
 }
 
+func createDockerWatchdog(ctx context.Context, execUI ui.ExecUI) *watchdog.WatchDog {
+	return watchdog.New(dockerWatchDuration, func() {
+		_, err := docker.NCPU(ctx)
+		if err != nil {
+			execUI.DockerWatchDogWarning(errors.Wrap(err, "docker watchdog"))
+		}
+	})
+}
+
 // executeBatchSpec performs all the steps required to upload the batch spec to
 // Sourcegraph, including execution as needed and applying the resulting batch
 // spec if specified.
-func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOpts) (err error) {
+func executeBatchSpec(ctx context.Context, opts executeBatchSpecOpts) (err error) {
+	var execUI ui.ExecUI
+	if opts.flags.textOnly {
+		execUI = &ui.JSONLines{}
+	} else {
+		out := output.NewOutput(os.Stderr, output.OutputOpts{Verbose: *verbose})
+		execUI = &ui.TUI{Out: out}
+	}
+
+	w := createDockerWatchdog(ctx, execUI)
+	go w.Start()
+
 	defer func() {
+		w.Stop()
 		if err != nil {
-			ui.ExecutionError(err)
+			execUI.ExecutionError(err)
 		}
 	}()
 
@@ -278,9 +290,19 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 		Client: opts.client,
 	})
 
+	ffs, err := svc.DetermineFeatureFlags(ctx)
+	if err != nil {
+		return err
+	}
+
+	// Once we know about feature flags, reconfigure the UI if needed.
+	if opts.flags.textOnly && ffs.BinaryDiffs {
+		execUI = &ui.JSONLines{BinaryDiffs: true}
+	}
+
 	imageCache := docker.NewImageCache()
 
-	if err := svc.DetermineFeatureFlags(ctx); err != nil {
+	if err := validateSourcegraphVersionConstraint(ctx, ffs); err != nil {
 		return err
 	}
 
@@ -288,9 +310,13 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 		return err
 	}
 
-	// In the past, we checked `docker version`, but now we retrieve the number
-	// of CPUs, since we need that anyway and it performs the same check (is
-	// Docker working _at all_?).
+	// In the past, we relied on `getBatchParallelism` to ascertain if docker is running,
+	// however, we don't always check for the number of CPUs (especially when the -j parallelis)
+	// flag is passed. This is a more explicit check to confirm docker is working.
+	if err := docker.CheckVersion(ctx); err != nil {
+		return err
+	}
+
 	parallelism, err := getBatchParallelism(ctx, opts.flags.parallelism)
 	if err != nil {
 		return err
@@ -319,12 +345,12 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 	}
 
 	// Parse flags and build up our service and executor options.
-	ui.ParsingBatchSpec()
-	batchSpec, rawSpec, err := parseBatchSpec(ctx, opts.file, svc, false)
+	execUI.ParsingBatchSpec()
+	batchSpec, batchSpecDir, rawSpec, err := parseBatchSpec(ctx, opts.file, svc)
 	if err != nil {
 		var multiErr errors.MultiError
 		if errors.As(err, &multiErr) {
-			ui.ParsingBatchSpecFailure(multiErr)
+			execUI.ParsingBatchSpecFailure(multiErr)
 			return cmderrors.ExitCode(2, nil)
 		} else {
 			// This shouldn't happen; let's just punt and let the normal
@@ -332,32 +358,32 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 			return err
 		}
 	}
-	ui.ParsingBatchSpecSuccess()
+	execUI.ParsingBatchSpecSuccess()
 
-	ui.ResolvingNamespace()
+	execUI.ResolvingNamespace()
 	namespace, err := svc.ResolveNamespace(ctx, opts.flags.namespace)
 	if err != nil {
 		return err
 	}
-	ui.ResolvingNamespaceSuccess(namespace.ID)
+	execUI.ResolvingNamespaceSuccess(namespace.ID)
 
 	var workspaceCreator workspace.Creator
 
 	if len(batchSpec.Steps) > 0 {
-		ui.PreparingContainerImages()
+		execUI.PreparingContainerImages()
 		images, err := svc.EnsureDockerImages(
 			ctx,
 			imageCache,
 			batchSpec.Steps,
 			parallelism,
-			ui.PreparingContainerImagesProgress,
+			execUI.PreparingContainerImagesProgress,
 		)
 		if err != nil {
 			return err
 		}
-		ui.PreparingContainerImagesSuccess()
+		execUI.PreparingContainerImagesSuccess()
 
-		ui.DeterminingWorkspaceCreatorType()
+		execUI.DeterminingWorkspaceCreatorType()
 		var typ workspace.CreatorType
 		workspaceCreator, typ = workspace.NewCreator(ctx, opts.flags.workspace, opts.flags.cacheDir, opts.flags.tempDir, images)
 		if typ == workspace.CreatorTypeVolume {
@@ -367,52 +393,21 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 				return err
 			}
 		}
-		ui.DeterminingWorkspaceCreatorTypeSuccess(typ)
+		execUI.DeterminingWorkspaceCreatorTypeSuccess(typ)
 	}
 
-	var (
-		repos      []*graphql.Repository
-		workspaces []service.RepoWorkspace
-	)
-	if svc.Features().ServerSideWorkspaceResolution {
-		ui.ResolvingRepositories()
-		var err error
-		workspaces, repos, err = svc.ResolveWorkspacesForBatchSpec(ctx, batchSpec, opts.flags.allowUnsupported, opts.flags.allowIgnored)
-		if err != nil {
-			if repoSet, ok := err.(batches.UnsupportedRepoSet); ok {
-				ui.ResolvingRepositoriesDone(repos, repoSet, nil)
-			} else if repoSet, ok := err.(batches.IgnoredRepoSet); ok {
-				ui.ResolvingRepositoriesDone(repos, nil, repoSet)
-			} else {
-				return errors.Wrap(err, "resolving repositories server-side")
-			}
+	execUI.DeterminingWorkspaces()
+	workspaces, repos, err := svc.ResolveWorkspacesForBatchSpec(ctx, batchSpec, opts.flags.allowUnsupported, opts.flags.allowIgnored)
+	if err != nil {
+		if repoSet, ok := err.(batches.UnsupportedRepoSet); ok {
+			execUI.DeterminingWorkspacesSuccess(len(workspaces), len(repos), repoSet, nil)
+		} else if repoSet, ok := err.(batches.IgnoredRepoSet); ok {
+			execUI.DeterminingWorkspacesSuccess(len(workspaces), len(repos), nil, repoSet)
 		} else {
-			ui.ResolvingRepositoriesDone(repos, nil, nil)
+			return errors.Wrap(err, "resolving repositories")
 		}
-
-		ui.DeterminingWorkspaces()
-		ui.DeterminingWorkspacesSuccess(len(workspaces))
 	} else {
-		ui.ResolvingRepositories()
-		repos, err = svc.ResolveRepositories(ctx, batchSpec, opts.flags.allowUnsupported, opts.flags.allowIgnored)
-		if err != nil {
-			if repoSet, ok := err.(batches.UnsupportedRepoSet); ok {
-				ui.ResolvingRepositoriesDone(repos, repoSet, nil)
-			} else if repoSet, ok := err.(batches.IgnoredRepoSet); ok {
-				ui.ResolvingRepositoriesDone(repos, nil, repoSet)
-			} else {
-				return errors.Wrap(err, "resolving repositories")
-			}
-		} else {
-			ui.ResolvingRepositoriesDone(repos, nil, nil)
-		}
-
-		ui.DeterminingWorkspaces()
-		workspaces, err = svc.DetermineWorkspaces(ctx, repos, batchSpec)
-		if err != nil {
-			return err
-		}
-		ui.DeterminingWorkspacesSuccess(len(workspaces))
+		execUI.DeterminingWorkspacesSuccess(len(workspaces), len(repos), nil, nil)
 	}
 
 	archiveRegistry := repozip.NewArchiveRegistry(opts.client, opts.flags.cacheDir, opts.flags.cleanArchives)
@@ -425,20 +420,21 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 				Creator:             workspaceCreator,
 				EnsureImage:         imageCache.Ensure,
 				Parallelism:         parallelism,
+				WorkingDirectory:    batchSpecDir,
 				Timeout:             opts.flags.timeout,
 				TempDir:             opts.flags.tempDir,
 				GlobalEnv:           os.Environ(),
-				IsRemote:            false,
+				ForceRoot:           opts.flags.runAsRoot,
+				BinaryDiffs:         ffs.BinaryDiffs,
 			},
-			Features:  svc.Features(),
-			Logger:    logManager,
-			Cache:     executor.NewDiskCache(opts.flags.cacheDir),
-			GlobalEnv: os.Environ(),
-			IsRemote:  false,
+			Logger:      logManager,
+			Cache:       executor.NewDiskCache(opts.flags.cacheDir),
+			BinaryDiffs: ffs.BinaryDiffs,
+			GlobalEnv:   os.Environ(),
 		},
 	)
 
-	ui.CheckingCache()
+	execUI.CheckingCache()
 	tasks := svc.BuildTasks(
 		&template.BatchChangeAttributes{
 			Name:        batchSpec.Name,
@@ -463,9 +459,9 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 			return err
 		}
 	}
-	ui.CheckingCacheSuccess(len(specs), len(uncachedTasks))
+	execUI.CheckingCacheSuccess(len(specs), len(uncachedTasks))
 
-	taskExecUI := ui.ExecutingTasks(*verbose, parallelism)
+	taskExecUI := execUI.ExecutingTasks(*verbose, parallelism)
 	freshSpecs, logFiles, execErr := coord.ExecuteAndBuildSpecs(ctx, batchSpec, uncachedTasks, taskExecUI)
 	// Add external changeset specs.
 	importedSpecs, importErr := svc.CreateImportChangesetSpecs(ctx, batchSpec)
@@ -482,7 +478,7 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 		if err == nil {
 			taskExecUI.Success()
 		} else {
-			ui.ExecutingTasksSkippingErrors(err)
+			execUI.ExecutingTasksSkippingErrors(err)
 		}
 	} else {
 		if err != nil {
@@ -492,7 +488,7 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 	}
 
 	if len(logFiles) > 0 && opts.flags.keepLogs {
-		ui.LogFilesKept(logFiles)
+		execUI.LogFilesKept(logFiles)
 	}
 
 	specs = append(specs, freshSpecs...)
@@ -506,7 +502,7 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 	ids := make([]graphql.ChangesetSpecID, len(specs))
 
 	if len(specs) > 0 {
-		ui.UploadingChangesetSpecs(len(specs))
+		execUI.UploadingChangesetSpecs(len(specs))
 
 		for i, spec := range specs {
 			id, err := svc.CreateChangesetSpec(ctx, spec)
@@ -514,33 +510,50 @@ func executeBatchSpec(ctx context.Context, ui ui.ExecUI, opts executeBatchSpecOp
 				return err
 			}
 			ids[i] = id
-			ui.UploadingChangesetSpecsProgress(i+1, len(specs))
+			execUI.UploadingChangesetSpecsProgress(i+1, len(specs))
 		}
 
-		ui.UploadingChangesetSpecsSuccess(ids)
+		execUI.UploadingChangesetSpecsSuccess(ids)
 	} else if len(repos) == 0 {
-		ui.NoChangesetSpecs()
+		execUI.NoChangesetSpecs()
 	}
 
-	ui.CreatingBatchSpec()
+	execUI.CreatingBatchSpec()
 	id, url, err := svc.CreateBatchSpec(ctx, namespace.ID, rawSpec, ids)
 	if err != nil {
-		return ui.CreatingBatchSpecError(err)
+		return execUI.CreatingBatchSpecError(err)
 	}
 	previewURL := cfg.Endpoint + url
-	ui.CreatingBatchSpecSuccess(previewURL)
+	execUI.CreatingBatchSpecSuccess(previewURL)
+
+	hasWorkspaceFiles := false
+	for _, step := range batchSpec.Steps {
+		if len(step.Mount) > 0 {
+			hasWorkspaceFiles = true
+			break
+		}
+	}
+	if hasWorkspaceFiles {
+		execUI.UploadingWorkspaceFiles()
+		if err = svc.UploadBatchSpecWorkspaceFiles(ctx, batchSpecDir, string(id), batchSpec.Steps); err != nil {
+			// Since failing to upload workspace files should not stop processing, just warn
+			execUI.UploadingWorkspaceFilesWarning(errors.Wrap(err, "uploading workspace files"))
+		} else {
+			execUI.UploadingWorkspaceFilesSuccess()
+		}
+	}
 
 	if !opts.applyBatchSpec {
-		ui.PreviewBatchSpec(previewURL)
+		execUI.PreviewBatchSpec(previewURL)
 		return
 	}
 
-	ui.ApplyingBatchSpec()
+	execUI.ApplyingBatchSpec()
 	batch, err := svc.ApplyBatchChange(ctx, id)
 	if err != nil {
 		return err
 	}
-	ui.ApplyingBatchSpecSuccess(cfg.Endpoint + batch.URL)
+	execUI.ApplyingBatchSpecSuccess(cfg.Endpoint + batch.URL)
 
 	return nil
 }
@@ -556,13 +569,10 @@ func setReadDeadlineOnCancel(ctx context.Context, f *os.File) {
 
 // parseBatchSpec parses and validates the given batch spec. If the spec has
 // validation errors, they are returned.
-//
-// isRemote argument is a temporary argument used to determine if the batch spec is being parsed for remote
-// (server-side) processing. Remote processing does not support mounts yet.
-func parseBatchSpec(ctx context.Context, file string, svc *service.Service, isRemote bool) (*batcheslib.BatchSpec, string, error) {
+func parseBatchSpec(ctx context.Context, file string, svc *service.Service) (*batcheslib.BatchSpec, string, string, error) {
 	f, err := batchOpenFileFlag(file)
 	if err != nil {
-		return nil, "", err
+		return nil, "", "", err
 	}
 	defer f.Close()
 
@@ -574,32 +584,41 @@ func parseBatchSpec(ctx context.Context, file string, svc *service.Service, isRe
 
 	data, err := io.ReadAll(f)
 	if err != nil {
-		return nil, "", errors.Wrap(err, "reading batch spec")
+		return nil, "", "", errors.Wrap(err, "reading batch spec")
 	}
 
+	dir, err := getBatchSpecDirectory(file)
+	if err != nil {
+		return nil, "", "", errors.Wrap(err, "batch spec path")
+	}
+
+	spec, err := svc.ParseBatchSpec(dir, data)
+	return spec, dir, string(data), err
+}
+
+func getBatchSpecDirectory(file string) (string, error) {
 	var workingDirectory string
+	var err error
 	// if the batch spec is being provided via standard input, set the working directory to the current directory
 	if file == "" || file == "-" {
 		workingDirectory, err = os.Getwd()
 		if err != nil {
-			return nil, "", errors.Wrap(err, "batch spec path")
+			return "", errors.Wrap(err, "batch spec path")
 		}
 	} else {
 		p, err := filepath.Abs(file)
 		if err != nil {
-			return nil, "", errors.Wrap(err, "batch spec path")
+			return "", errors.Wrap(err, "batch spec path")
 		}
 		workingDirectory = filepath.Dir(p)
 	}
-
-	spec, err := svc.ParseBatchSpec(workingDirectory, data, isRemote)
-	return spec, string(data), err
+	return workingDirectory, nil
 }
 
 func checkExecutable(cmd string, args ...string) error {
 	if err := exec.Command(cmd, args...).Run(); err != nil {
 		return fmt.Errorf(
-			"failed to execute \"%s %s\":\n\t%s\n\n'src batch' require %q to be available.",
+			"failed to execute \"%s %s\":\n\t%s\n\n'src batch' requires %q to be available.",
 			cmd,
 			strings.Join(args, " "),
 			err,
@@ -635,3 +654,10 @@ func getBatchParallelism(ctx context.Context, flag int) (int, error) {
 
 	return docker.NCPU(ctx)
 }
+
+func validateSourcegraphVersionConstraint(ctx context.Context, ffs *batches.FeatureFlags) error {
+	if ffs.Sourcegraph40 {
+		return nil
+	}
+	return errors.Newf("\n\n * Warning:\n This version of src-cli requires Sourcegraph version 4.0 or newer. If you're not on Sourcegraph 4.0 or newer, please use the 3.x release of src-cli that corresponds to your Sourcegraph version.\n\n")
+}
