@@ -10,16 +10,17 @@ import (
 	"path/filepath"
 	"time"
 
-	"github.com/sourcegraph/sourcegraph/lib/errors"
-
 	"github.com/sourcegraph/src-cli/internal/batches/docker"
-	"github.com/sourcegraph/src-cli/internal/batches/executor"
-	"github.com/sourcegraph/src-cli/internal/batches/graphql"
 	"github.com/sourcegraph/src-cli/internal/batches/log"
 	"github.com/sourcegraph/src-cli/internal/batches/repozip"
+	"github.com/sourcegraph/src-cli/internal/batches/workspace"
+
+	"github.com/sourcegraph/sourcegraph/lib/errors"
+
+	"github.com/sourcegraph/src-cli/internal/batches/executor"
+	"github.com/sourcegraph/src-cli/internal/batches/graphql"
 	"github.com/sourcegraph/src-cli/internal/batches/service"
 	"github.com/sourcegraph/src-cli/internal/batches/ui"
-	"github.com/sourcegraph/src-cli/internal/batches/workspace"
 	"github.com/sourcegraph/src-cli/internal/cmderrors"
 
 	batcheslib "github.com/sourcegraph/sourcegraph/lib/batches"
@@ -30,18 +31,24 @@ const (
 )
 
 type executorModeFlags struct {
-	timeout time.Duration
-	file    string
-	tempDir string
-	repoDir string
+	timeout           time.Duration
+	file              string
+	runAsImageUser    bool
+	tempDir           string
+	repoDir           string
+	workspaceFilesDir string
+	binaryDiffs       bool
 }
 
 func newExecutorModeFlags(flagSet *flag.FlagSet) (f *executorModeFlags) {
 	f = &executorModeFlags{}
 	flagSet.DurationVar(&f.timeout, "timeout", 60*time.Minute, "The maximum duration a single batch spec step can take.")
 	flagSet.StringVar(&f.file, "f", "", "The workspace execution input file to read.")
+	flagSet.BoolVar(&f.runAsImageUser, "run-as-image-user", false, "True to run step containers as the default image user; if false or omitted, containers are always run as root.")
 	flagSet.StringVar(&f.tempDir, "tmp", "", "Directory for storing temporary data.")
 	flagSet.StringVar(&f.repoDir, "repo", "", "Path of the checked out repo on disk.")
+	flagSet.StringVar(&f.workspaceFilesDir, "workspaceFiles", "", "Path of workspace files on disk.")
+	flagSet.BoolVar(&f.binaryDiffs, "binaryDiffs", false, "Whether to encode diffs as base64.")
 
 	return f
 }
@@ -69,7 +76,7 @@ github.com/sourcegraph/sourcegraph/lib/batches.
 
 Usage:
 
-    src batch exec -f FILE -repo DIR [command options]
+    src batch exec -f FILE -repo DIR -workspaceFiles DIR [command options]
 
 Examples:
 
@@ -116,7 +123,7 @@ Examples:
 }
 
 func executeBatchSpecInWorkspaces(ctx context.Context, flags *executorModeFlags) (err error) {
-	ui := &ui.JSONLines{}
+	ui := &ui.JSONLines{BinaryDiffs: flags.binaryDiffs}
 
 	// Ensure the temp dir exists.
 	tempDir := flags.tempDir
@@ -140,6 +147,12 @@ func executeBatchSpecInWorkspaces(ctx context.Context, flags *executorModeFlags)
 		}
 	}
 
+	// Grab the absolute path to the workspace files contents.
+	workspaceFilesDir, err := filepath.Abs(flags.workspaceFilesDir)
+	if err != nil {
+		return errors.Wrap(err, "getting absolute path for workspace files dir")
+	}
+
 	// Test if git is available.
 	if err := checkExecutable("git", "version"); err != nil {
 		return err
@@ -176,34 +189,37 @@ func executeBatchSpecInWorkspaces(ctx context.Context, flags *executorModeFlags)
 	}
 	ui.PreparingContainerImagesSuccess()
 
-	// Empty for now until we support secrets or env var settings in SSBC.
-	globalEnv := []string{}
-	isRemote := true
-
 	// Set up the execution UI.
 	taskExecUI := ui.ExecutingTasks(false, 1)
 	taskExecUI.Start([]*executor.Task{task})
 	taskExecUI.TaskStarted(task)
 
+	// Pass the os.Environ to run steps to allow access to the secrets set
+	// in the executor environment.
+	// The executor runtime takes care of not forwarding any sensitive secrets
+	// from the host, so this is safe.
+	globalEnv := os.Environ()
+
 	opts := &executor.RunStepsOpts{
 		Logger:      &log.NoopTaskLogger{},
 		WC:          workspace.NewExecutorWorkspaceCreator(tempDir, repoDir),
 		EnsureImage: imageCache.Ensure,
 		Task:        task,
 		// TODO: Should be slightly less than the executor timeout. Can we somehow read that?
-		Timeout:   flags.timeout,
-		TempDir:   tempDir,
-		GlobalEnv: globalEnv,
-		// Temporarily prevent the ability to sending a batch spec with a mount for server-side processing.
-		IsRemote:    isRemote,
-		RepoArchive: &repozip.NoopArchive{},
-		UI:          taskExecUI.StepsExecutionUI(task),
+		Timeout:          flags.timeout,
+		TempDir:          tempDir,
+		WorkingDirectory: workspaceFilesDir,
+		GlobalEnv:        globalEnv,
+		RepoArchive:      &repozip.NoopArchive{},
+		UI:               taskExecUI.StepsExecutionUI(task),
+		ForceRoot:        !flags.runAsImageUser,
+		BinaryDiffs:      flags.binaryDiffs,
 	}
 	results, err := executor.RunSteps(ctx, opts)
 
 	// Write all step cache results for all results.
 	for _, stepRes := range results {
-		cacheKey := task.CacheKey(globalEnv, isRemote, stepRes.StepIndex)
+		cacheKey := task.CacheKey(globalEnv, workspaceFilesDir, stepRes.StepIndex)
 		k, err := cacheKey.Key()
 		if err != nil {
 			return errors.Wrap(err, "calculating step cache key")
