@@ -48,7 +48,12 @@ Examples:
 			Client: cfg.apiClient(apiFlags, flagSet.Output()),
 		})
 
-		if err := svc.DetermineFeatureFlags(ctx); err != nil {
+		ffs, err := svc.DetermineFeatureFlags(ctx)
+		if err != nil {
+			return err
+		}
+
+		if err := validateSourcegraphVersionConstraint(ctx, ffs); err != nil {
 			return err
 		}
 
