@@ -52,7 +52,12 @@ Examples:
 			Client: cfg.apiClient(flags.api, flagSet.Output()),
 		})
 
-		if err := svc.DetermineFeatureFlags(ctx); err != nil {
+		ffs, err := svc.DetermineFeatureFlags(ctx)
+		if err != nil {
+			return err
+		}
+
+		if err := validateSourcegraphVersionConstraint(ctx, ffs); err != nil {
 			return err
 		}
 
@@ -63,7 +68,7 @@ Examples:
 		// may as well validate it at the same time so we don't even have to go to
 		// the backend if it's invalid.
 		ui.ParsingBatchSpec()
-		spec, raw, err := parseBatchSpec(ctx, file, svc, true)
+		spec, batchSpecDir, raw, err := parseBatchSpec(ctx, file, svc)
 		if err != nil {
 			ui.ParsingBatchSpecFailure(err)
 			return err
@@ -79,7 +84,7 @@ Examples:
 		ui.ResolvingNamespaceSuccess(namespace.ID)
 
 		ui.SendingBatchChange()
-		batchChangeName, err := svc.UpsertBatchChange(ctx, spec.Name, namespace.ID)
+		batchChangeID, batchChangeName, err := svc.UpsertBatchChange(ctx, spec.Name, namespace.ID)
 		if err != nil {
 			return err
 		}
@@ -93,18 +98,35 @@ Examples:
 			flags.allowIgnored,
 			flags.allowUnsupported,
 			flags.clearCache,
+			batchChangeID,
 		)
 		if err != nil {
 			return err
 		}
 		ui.SendingBatchSpecSuccess()
 
+		hasWorkspaceFiles := false
+		for _, step := range spec.Steps {
+			if len(step.Mount) > 0 {
+				hasWorkspaceFiles = true
+				break
+			}
+		}
+		if hasWorkspaceFiles {
+			ui.UploadingWorkspaceFiles()
+			if err = svc.UploadBatchSpecWorkspaceFiles(ctx, batchSpecDir, batchSpecID, spec.Steps); err != nil {
+				return err
+			}
+			ui.UploadingWorkspaceFilesSuccess()
+		}
+
 		// Wait for the workspaces to be resolved.
 		ui.ResolvingWorkspaces()
 		ticker := time.NewTicker(1 * time.Second)
 		defer ticker.Stop()
+		var res *service.BatchSpecWorkspaceResolution
 		for range ticker.C {
-			res, err := svc.GetBatchSpecWorkspaceResolution(ctx, batchSpecID)
+			res, err = svc.GetBatchSpecWorkspaceResolution(ctx, batchSpecID)
 			if err != nil {
 				return err
 			}
@@ -115,7 +137,7 @@ Examples:
 				break
 			}
 		}
-		ui.ResolvingWorkspacesSuccess()
+		ui.ResolvingWorkspacesSuccess(res.Workspaces.TotalCount)
 
 		// We have to enqueue this for execution with a separate operation.
 		//
