@@ -63,7 +63,12 @@ Examples:
 			Client: cfg.apiClient(apiFlags, flagSet.Output()),
 		})
 
-		if err := svc.DetermineFeatureFlags(ctx); err != nil {
+		ffs, err := svc.DetermineFeatureFlags(ctx)
+		if err != nil {
+			return err
+		}
+
+		if err := validateSourcegraphVersionConstraint(ctx, ffs); err != nil {
 			ui.ExecutionError(err)
 			return err
 		}
@@ -73,7 +78,7 @@ Examples:
 			return err
 		}
 
-		if _, _, err := parseBatchSpec(ctx, file, svc, false); err != nil {
+		if _, _, _, err := parseBatchSpec(ctx, file, svc); err != nil {
 			ui.ParsingBatchSpecFailure(err)
 			return err
 		}
