@@ -23,7 +23,9 @@ import (
 
 var _ ExecUI = &JSONLines{}
 
-type JSONLines struct{}
+type JSONLines struct {
+	BinaryDiffs bool
+}
 
 func (ui *JSONLines) ParsingBatchSpec() {
 	logOperationStart(batcheslib.LogEventOperationParsingBatchSpec, &batcheslib.ParsingBatchSpecMetadata{})
@@ -66,22 +68,16 @@ func (ui *JSONLines) DeterminingWorkspaceCreatorTypeSuccess(wt workspace.Creator
 	logOperationSuccess(batcheslib.LogEventOperationDeterminingWorkspaceType, &batcheslib.DeterminingWorkspaceTypeMetadata{Type: t})
 }
 
-func (ui *JSONLines) ResolvingRepositories() {
-	logOperationStart(batcheslib.LogEventOperationResolvingRepositories, &batcheslib.ResolvingRepositoriesMetadata{})
-}
-func (ui *JSONLines) ResolvingRepositoriesDone(repos []*graphql.Repository, unsupported batches.UnsupportedRepoSet, ignored batches.IgnoredRepoSet) {
-	logOperationSuccess(batcheslib.LogEventOperationResolvingRepositories, &batcheslib.ResolvingRepositoriesMetadata{
-		Unsupported: len(unsupported),
-		Ignored:     len(ignored),
-		Count:       len(repos),
-	})
-}
-
 func (ui *JSONLines) DeterminingWorkspaces() {
 	logOperationStart(batcheslib.LogEventOperationDeterminingWorkspaces, &batcheslib.DeterminingWorkspacesMetadata{})
 }
-func (ui *JSONLines) DeterminingWorkspacesSuccess(num int) {
-	logOperationSuccess(batcheslib.LogEventOperationDeterminingWorkspaces, &batcheslib.DeterminingWorkspacesMetadata{Count: num})
+func (ui *JSONLines) DeterminingWorkspacesSuccess(workspacesCount, reposCount int, unsupported batches.UnsupportedRepoSet, ignored batches.IgnoredRepoSet) {
+	logOperationSuccess(batcheslib.LogEventOperationDeterminingWorkspaces, &batcheslib.DeterminingWorkspacesMetadata{
+		Unsupported:    len(unsupported),
+		Ignored:        len(ignored),
+		RepoCount:      reposCount,
+		WorkspaceCount: workspacesCount,
+	})
 }
 
 func (ui *JSONLines) CheckingCache() {
@@ -95,7 +91,9 @@ func (ui *JSONLines) CheckingCacheSuccess(cachedSpecsFound int, tasksToExecute i
 }
 
 func (ui *JSONLines) ExecutingTasks(_ bool, _ int) executor.TaskExecutionUI {
-	return &taskExecutionJSONLines{}
+	return &taskExecutionJSONLines{
+		binaryDiffs: ui.BinaryDiffs,
+	}
 }
 
 func (ui *JSONLines) ExecutingTasksSkippingErrors(err error) {
@@ -179,8 +177,17 @@ func (ui *JSONLines) WriteAfterStepResult(key string, value execution.AfterStepR
 	})
 }
 
+func (ui *JSONLines) DockerWatchDogWarning(err error) {
+	message := fmt.Sprintf(`It seems your Docker engine might be frozen.
+If there's no progress in the next couple minutes, you may want to try restarting Docker and running the command again.
+Error: %s
+`, err.Error())
+	logOperationFailure(batcheslib.LogEventOperationDockerWatchDog, &batcheslib.DockerWatchDogMetadata{Error: message})
+}
+
 type taskExecutionJSONLines struct {
-	linesTasks map[*executor.Task]batcheslib.JSONLinesTask
+	linesTasks  map[*executor.Task]batcheslib.JSONLinesTask
+	binaryDiffs bool
 }
 
 // seededRand is used in randomID() to generate a "random" number.
@@ -262,7 +269,8 @@ func (ui *taskExecutionJSONLines) StepsExecutionUI(task *executor.Task) executor
 }
 
 type stepsExecutionJSONLines struct {
-	linesTask *batcheslib.JSONLinesTask
+	linesTask   *batcheslib.JSONLinesTask
+	binaryDiffs bool
 }
 
 const stepFlushDuration = 500 * time.Millisecond
@@ -300,7 +308,14 @@ func (ui *stepsExecutionJSONLines) StepPreparingFailed(step int, err error) {
 }
 
 func (ui *stepsExecutionJSONLines) StepStarted(step int, runScript string, env map[string]string) {
-	logOperationStart(batcheslib.LogEventOperationTaskStep, &batcheslib.TaskStepMetadata{TaskID: ui.linesTask.ID, Step: step, Env: env})
+	logOperationStart(
+		batcheslib.LogEventOperationTaskStep,
+		&batcheslib.TaskStepMetadata{
+			Version: version(ui.binaryDiffs),
+			TaskID:  ui.linesTask.ID,
+			Step:    step,
+			Env:     env,
+		})
 }
 
 func (ui *stepsExecutionJSONLines) StepOutputWriter(ctx context.Context, task *executor.Task, step int) executor.StepOutputWriter {
@@ -308,19 +323,21 @@ func (ui *stepsExecutionJSONLines) StepOutputWriter(ctx context.Context, task *e
 		logOperationProgress(
 			batcheslib.LogEventOperationTaskStep,
 			&batcheslib.TaskStepMetadata{
-				TaskID: ui.linesTask.ID,
-				Step:   step,
-				Out:    data,
+				Version: version(ui.binaryDiffs),
+				TaskID:  ui.linesTask.ID,
+				Step:    step,
+				Out:     data,
 			},
 		)
 	}
 	return NewIntervalProcessWriter(ctx, stepFlushDuration, sink)
 }
 
-func (ui *stepsExecutionJSONLines) StepFinished(step int, diff string, changes git.Changes, outputs map[string]interface{}) {
+func (ui *stepsExecutionJSONLines) StepFinished(step int, diff []byte, changes git.Changes, outputs map[string]interface{}) {
 	logOperationSuccess(
 		batcheslib.LogEventOperationTaskStep,
 		&batcheslib.TaskStepMetadata{
+			Version: version(ui.binaryDiffs),
 			TaskID:  ui.linesTask.ID,
 			Step:    step,
 			Diff:    diff,
@@ -333,6 +350,7 @@ func (ui *stepsExecutionJSONLines) StepFailed(step int, err error, exitCode int)
 	logOperationFailure(
 		batcheslib.LogEventOperationTaskStep,
 		&batcheslib.TaskStepMetadata{
+			Version:  version(ui.binaryDiffs),
 			TaskID:   ui.linesTask.ID,
 			Step:     step,
 			Error:    err.Error(),
@@ -341,6 +359,18 @@ func (ui *stepsExecutionJSONLines) StepFailed(step int, err error, exitCode int)
 	)
 }
 
+func (ui *JSONLines) UploadingWorkspaceFiles() {
+	// No workspace file upload required for executor mode.
+}
+
+func (ui *JSONLines) UploadingWorkspaceFilesWarning(err error) {
+	// No workspace file upload required for executor mode.
+}
+
+func (ui *JSONLines) UploadingWorkspaceFilesSuccess() {
+	// No workspace file upload required for executor mode.
+}
+
 func logOperationStart(op batcheslib.LogEventOperation, metadata interface{}) {
 	logEvent(batcheslib.LogEvent{Operation: op, Status: batcheslib.LogEventStatusStarted, Metadata: metadata})
 }
@@ -364,3 +394,10 @@ func logEvent(e batcheslib.LogEvent) {
 		fmt.Fprintln(os.Stderr, err)
 	}
 }
+
+func version(binaryDiffs bool) int {
+	if binaryDiffs {
+		return 2
+	}
+	return 1
+}
