@@ -3,9 +3,11 @@ package ui
 import (
 	"context"
 	"fmt"
+	"math"
 	"os/exec"
 
 	"github.com/neelance/parallel"
+
 	"github.com/sourcegraph/sourcegraph/lib/errors"
 	"github.com/sourcegraph/sourcegraph/lib/output"
 
@@ -21,6 +23,8 @@ var (
 	batchPendingColor = output.StylePending
 	batchSuccessColor = output.StyleSuccess
 	batchSuccessEmoji = output.EmojiSuccess
+	batchWarningColor = output.StyleWarning
+	batchWarningEmoji = output.EmojiWarning
 )
 
 var _ ExecUI = &TUI{}
@@ -93,33 +97,51 @@ func (ui *TUI) DeterminingWorkspaceCreatorTypeSuccess(wt workspace.CreatorType)
 	batchCompletePending(ui.pending, "Set workspace type")
 }
 
-func (ui *TUI) ResolvingRepositories() {
-	ui.pending = batchCreatePending(ui.Out, "Resolving repositories")
+func (ui *TUI) DeterminingWorkspaces() {
+	ui.pending = batchCreatePending(ui.Out, "Determining workspaces")
 }
-func (ui *TUI) ResolvingRepositoriesDone(repos []*graphql.Repository, unsupported batches.UnsupportedRepoSet, ignored batches.IgnoredRepoSet) {
-	batchCompletePending(ui.pending, fmt.Sprintf("Resolved %d repositories", len(repos)))
 
-	if unsupported != nil && len(unsupported) != 0 {
+func (ui *TUI) DeterminingWorkspacesSuccess(workspacesCount, reposCount int, unsupported batches.UnsupportedRepoSet, ignored batches.IgnoredRepoSet) {
+	batchCompletePending(ui.pending, fmt.Sprintf("Resolved %d workspaces from %d repositories", workspacesCount, reposCount))
+
+	if len(unsupported) != 0 {
 		block := ui.Out.Block(output.Line(" ", output.StyleWarning, "Some repositories are hosted on unsupported code hosts and will be skipped. Use the -allow-unsupported flag to avoid skipping them."))
 		for repo := range unsupported {
 			block.Write(repo.Name)
 		}
 		block.Close()
-	} else if ignored != nil && len(ignored) != 0 {
+	} else if len(ignored) != 0 {
 		block := ui.Out.Block(output.Line(" ", output.StyleWarning, "The repositories listed below contain .batchignore files and will be skipped. Use the -force-override-ignore flag to avoid skipping them."))
 		for repo := range ignored {
 			block.Write(repo.Name)
 		}
 		block.Close()
 	}
+
+	ui.maybeWorkspaceCountWarning(workspacesCount, 500)
 }
 
-func (ui *TUI) DeterminingWorkspaces() {
-	ui.pending = batchCreatePending(ui.Out, "Determining workspaces")
-}
+func (ui *TUI) maybeWorkspaceCountWarning(count, limit int) {
+	if count > limit {
+		block := ui.Out.Block(output.Linef(
+			"⚠️", output.StyleWarning,
+			"Batch changes with more than %d workspaces may be unwieldy to manage.",
+			limit,
+		))
 
-func (ui *TUI) DeterminingWorkspacesSuccess(num int) {
-	batchCompletePending(ui.pending, fmt.Sprintf("Found %d workspaces with steps to execute", num))
+		for _, line := range []string{
+			"We're working on providing more filtering options, and you can continue with",
+			fmt.Sprintf(
+				"this batch change if you want, but you may want to break it into %d or more",
+				int(math.Ceil(float64(count)/float64(limit))),
+			),
+			"batch changes if you can.",
+		} {
+			block.WriteLine(output.Line("", output.StyleSuggestion, line))
+		}
+
+		block.Close()
+	}
 }
 
 func (ui *TUI) CheckingCache() {
@@ -199,6 +221,10 @@ func (ui *TUI) CreatingBatchSpecError(err error) error {
 	return prettyPrintBatchUnlicensedError(ui.Out, err)
 }
 
+func (ui *TUI) DockerWatchDogWarning(err error) {
+	dockerWatchDogWarning(ui.Out, err)
+}
+
 func (ui *TUI) PreviewBatchSpec(batchSpecURL string) {
 	ui.Out.Write("")
 	block := ui.Out.Block(output.Line(batchSuccessEmoji, batchSuccessColor, "To preview or apply the batch spec, go to:"))
@@ -239,12 +265,25 @@ func (ui *TUI) SendingBatchSpecSuccess() {
 	batchCompletePending(ui.pending, "Sending batch spec")
 }
 
+func (ui *TUI) UploadingWorkspaceFiles() {
+	ui.pending = batchCreatePending(ui.Out, "Uploading workspace files")
+}
+
+func (ui *TUI) UploadingWorkspaceFilesWarning(err error) {
+	batchCompleteWarning(ui.pending, err.Error())
+}
+
+func (ui *TUI) UploadingWorkspaceFilesSuccess() {
+	batchCompletePending(ui.pending, "Uploading workspace files")
+}
+
 func (ui *TUI) ResolvingWorkspaces() {
 	ui.pending = batchCreatePending(ui.Out, "Resolving workspaces")
 }
 
-func (ui *TUI) ResolvingWorkspacesSuccess() {
+func (ui *TUI) ResolvingWorkspacesSuccess(workspacesCount int) {
 	batchCompletePending(ui.pending, "Resolving workspaces")
+	ui.maybeWorkspaceCountWarning(workspacesCount, 2000)
 }
 
 func (ui *TUI) ExecutingBatchSpec() {
@@ -282,14 +321,14 @@ func prettyPrintBatchUnlicensedError(out *output.Output, err error) error {
 				// verbose mode, but let the original error bubble up rather
 				// than this one.
 				out.Verbosef("Unexpected error parsing the GraphQL error: %v", cerr)
-			} else if code == "ErrCampaignsUnlicensed" || code == "ErrBatchChangesUnlicensed" {
+			} else if code == "ErrBatchChangesUnlicensed" {
 				// OK, let's print a better message, then return an
 				// exitCodeError to suppress the normal automatic error block.
 				// Note that we have hand wrapped the output at 80 (printable)
 				// characters: having automatic wrapping some day would be nice,
 				// but this should be sufficient for now.
 				block := out.Block(output.Line("🪙", output.StyleWarning, "Batch Changes is a paid feature of Sourcegraph. All users can create sample"))
-				block.WriteLine(output.Linef("", output.StyleWarning, "batch changes with up to 5 changesets without a license. Contact Sourcegraph"))
+				block.WriteLine(output.Linef("", output.StyleWarning, "batch changes with up to 10 changesets without a license. Contact Sourcegraph"))
 				block.WriteLine(output.Linef("", output.StyleWarning, "sales at %shttps://about.sourcegraph.com/contact/sales/%s to obtain a trial", output.StyleSearchLink, output.StyleWarning))
 				block.WriteLine(output.Linef("", output.StyleWarning, "license."))
 				block.Write("")
@@ -410,3 +449,15 @@ func batchCreatePending(out *output.Output, message string) output.Pending {
 func batchCompletePending(p output.Pending, message string) {
 	p.Complete(output.Line(batchSuccessEmoji, batchSuccessColor, message))
 }
+
+func batchCompleteWarning(p output.Pending, message string) {
+	p.Complete(output.Line(batchWarningEmoji, batchWarningColor, message))
+}
+
+func dockerWatchDogWarning(out *output.Output, err error) {
+	block := out.Block(output.Line("🐳", output.StyleWarning, "It seems your Docker engine might be frozen."))
+	block.WriteLine(output.Line("", output.StyleWarning, "If there's no progress in the next couple minutes, you may want to try restarting Docker and running the command again."))
+	block.WriteLine(output.Linef("", output.StyleWarning, "Error: %s", err.Error()))
+	block.Write("")
+	block.Close()
+}
